%function [u,x,t] = spec_vdiffuse(X,T,N,f,nu,alpha)
%Performs the a a psuedo spectral method for the equation
%       u_t + u_x  = a(u_xx) + F(u), t > 0 and a < x < b
%       F(u) = (nu(x)u_x)_x - au_xx
%       u(0,x) = f(x),  u(a,x) = u(t,b)
% By treating a(u_xx) and implicit and F(u) as explicit and a first order 
% time approxiation and iterating in spectral space.
%
%INPUT
%   X = Space interval [a,b].
%   T = Time interval [0,t].
%   N = Space and time partition numbers,[space,time], Best if space = 2^n.
%   f = Function handle for initial condition for t=0.
%   nu = Function handle for diffusion coefficient.
%   alpha = bound for diffusion coefficient.
%
%OUTPUT
%   u = Approximation
%   x = Space decomposition
%   t = Time decomposition
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Debug
dbug = 1;
if dbug
    close all; clear all; clc;
    dbug = 1;
    X = [0,2*pi];
    T = [0,1];
    n = 2^4;
    m = 60;
    N = [n,m];
    nu = @(x) 2 - sin(x);
    f = @(x) 3./(5-4*cos(x));
    alpha = 1/2;
end

%% Initialization
a = X(1); b = X(2);
ta = T(1); tb = T(2);
n = N(1); m = N(2);

%% Time and space decompositions
t = linspace(ta,tb,m+1);
x = linspace(a,b,n+1);
x(n+1) = [];
dt = t(2) - t(1);

%% Compute using full spectral approximation.
u = zeros(n,m);
u(:,1) = f(x);
jVec = 2*pi/(b-a)*[0:n/2-1, 0, -n/2+1:-1];
jVec2 = 2*pi/(b-a)*[0:n/2-1, -n/2, -n/2+1:-1];

for K = 1:m
    fk = fft(u(:,K),n)';
    ux = 1i*jVec.*fk;
    nuux = fft(nu(x).*ifft(ux,n));
    auxx = -alpha*jVec2.^2.*fk;
    nuuxx = 1i*jVec.*nuux;
    F = (nuuxx - auxx);
    u(:,K+1) = ifft((1 + dt*alpha*jVec2.^2).\(fk + dt*F),n);
end

%% Fix endpoint
u = real(u);
u(n+1,:) = u(1,:);
x = [x b];

%% Plots for Debugging
if dbug
%     sol = solution(x,t,nu);
%     err = zeros(1,m);
%     for K = 1:m+1
%         err(K) = max(norm(u(:,K)-sol(:,K)));
%     end

%     figure(2)
%     surf(t,x,sol);
%     title('solution');

    figure(1)
    surf(t,x,u);
    title('approximation');

%     figure(3)
%     plot(t,err)
    
    dataM  = char('o', 'd','.','s','+','v');
    dataC  = char('b', 'm','g','k','r','c');
    figure(4)
    M = m;
    pgVec = floor([1 M/4 M/2 3*M/4 M]);
    for J = 1:5
        plot(x,u(:,pgVec(J)),'color',dataC(J));
        hold on
    end
    title('Time slices of approximation');
    legend('t=0','t=.25','t=.5','t=.75', 't=1')
    hold off

end